
subroutine CalcEnergy(Pos, ChargeList, M, L, rc, kappa, lambda, PEnergy, Dim, NAtom)
    implicit none
    integer, intent(in) :: M, Dim, NAtom
    real(8), intent(in), dimension(0:NAtom-1, 0:Dim-1) :: Pos
    real(8), intent(in), dimension(0:NAtom-1) :: ChargeList
    real(8), intent(in) :: L,rc, kappa, lambda
    real(8), intent(out) :: PEnergy
    real(8), parameter :: k = 3000., r0 = 1.
    real(8), dimension(Dim) :: rij, Posi
    real(8) :: d, d2, id2, id6, id12, sign
    real(8) :: rc2, Shift
    integer :: i, j
    PEnergy = 0.
    
    Shift = -4. * (rc**(-12)-rc**(-6))
    rc2 = rc * rc

    
    do i = 0, NAtom - 1
        !store Pos(i,:) in a temporary array for faster access in j loop
        Posi = Pos(i,:)
        do j = i + 1, NAtom - 1
            
            rij = Pos(j,:) - Posi
            rij = rij - L * dnint(rij / L)
            d = sqrt(sum(rij * rij))

            !Screened Coloumb Interaction
            sign = ChargeList(i)*ChargeList(j)
            PEnergy = PEnergy + sign*lambda/d*exp(-1.0*d*kappa)

            
            if ( j == i + 1 .and. sign == 0.0 .and. mod(j, M) > 0) then
            !this is a bonded interaction;
            !use the harmonic bond poential 
                d = sqrt(sum(rij * rij))
                PEnergy = PEnergy + k/2. * (d - r0) * (d - r0)
            
            else if ( j == i + 1 .and. sign == 1.0 .and. ChargeList(i) == 1 ) then
            !this is a bonded interaction;
            !use the harmonic bond poential 
                d = sqrt(sum(rij * rij))
                PEnergy = PEnergy + k/2. * (d - r0) * (d - r0)
                
            else
            !this is a nonbonded interaction
            !use LJ Potenetial
                d2 = sum(rij * rij)                
                if (d2 > rc2) then
                    cycle
                end if
                id2 = 1. / d2             !inverse squared distance
                id6 = id2 * id2 * id2     !inverse sixth distance
                id12 = id6 * id6          !inverse twelvth distance
                PEnergy = PEnergy + 4. * (id12 - id6) + Shift
            endif
        enddo
    enddo
end subroutine


subroutine CalcEnergyForces(Pos,ChargeList, M, L, rc, kappa, lambda, PEnergy, Forces, Dim, NAtom)
    implicit none
    integer, intent(in) :: M, Dim, NAtom
    real(8), intent(in), dimension(0:NAtom-1) :: ChargeList
    real(8), intent(in), dimension(0:NAtom-1, 0:Dim-1) :: Pos
    real(8), intent(in) :: L, rc, kappa, lambda
    real(8), intent(out) :: PEnergy
    real(8), intent(inout), dimension(0:NAtom-1, 0:Dim-1) :: Forces
!f2py intent(in, out, inplace) :: Forces
    real(8), parameter :: k = 3000., r0 = 1.
    real(8), dimension(Dim) :: rij, Fij, Posi
    real(8) :: d, d2, id2, id6, id12, sign
    real(8) :: rc2, Shift
    integer :: i, j
    
    PEnergy = 0.
    Forces = 0.
    
    Shift = -4. * (rc**(-12)-rc**(-6))
    rc2 = rc * rc

    do i = 0, NAtom - 1
        !store Pos(i,:) in a temporary array for faster access in j loop
        Posi = Pos(i,:)
        do j = i + 1, NAtom - 1
            
            rij = Pos(j,:) - Posi
            rij = rij - L * dnint(rij / L)
            d = sqrt(sum(rij * rij))

            !Screened Coloumb Interaction
            sign = ChargeList(i)*ChargeList(j)
            PEnergy = PEnergy + sign*lambda/d*exp(-1.0*d*kappa)
            Fij = rij * (-1.0*sign*lambda*(1.0/(d*d*d)+kappa/(d*d))*exp(-1.0*d*kappa))
            Forces(i,:) = Forces(i,:) + Fij
            Forces(j,:) = Forces(j,:) - Fij

            
            if ( j == i + 1 .and. sign == 0. .and. mod(j, M) > 0) then
            !this is a bonded interaction;
            !use the harmonic bond poential 
                d = sqrt(sum(rij * rij))
                PEnergy = PEnergy + k/2. * (d - r0) * (d - r0)
                Fij = rij * k * (1-r0/d)
                Forces(i,:) = Forces(i,:) + Fij
                Forces(j,:) = Forces(j,:) - Fij
                
            
            else if ( j == i + 1 .and. sign == 1. .and. ChargeList(j) /= -1. ) then
            !this is a bonded interaction;
            !use the harmonic bond poential 
                d = sqrt(sum(rij * rij))
                PEnergy = PEnergy + k/2. * (d - r0) * (d - r0)
                Fij = rij * k * (1-r0/d)
                Forces(i,:) = Forces(i,:) + Fij
                Forces(j,:) = Forces(j,:) - Fij
                
            else
            !this is a nonbonded interaction
            !use LJ Potenetial
                d2 = sum(rij * rij)                
                if (d2 > rc2) then
                    cycle
                end if
                id2 = 1. / d2             !inverse squared distance
                id6 = id2 * id2 * id2     !inverse sixth distance
                id12 = id6 * id6          !inverse twelvth distance
                PEnergy = PEnergy + 4. * (id12 - id6) + Shift
                Fij = rij * ((-48. * id12 + 24. * id6) * id2)
                Forces(i,:) = Forces(i,:) + Fij
                Forces(j,:) = Forces(j,:) - Fij
            endif
        enddo
    enddo
end subroutine


subroutine VVIntegrate(Pos, M, ChargeList, Vel, Accel, L, rc, kappa, lambda, dt, KEnergy, PEnergy, Dim, NAtom)
    implicit none
    integer, intent(in) :: M, Dim, NAtom
    real(8), intent(in) :: L, rc, kappa, lambda, dt
    real(8), intent(in), dimension(0:NAtom-1) :: ChargeList
    real(8), intent(inout), dimension(0:NAtom-1, 0:Dim-1) :: Pos, Vel, Accel
!f2py intent(in,out,inplace) :: Pos, Vel, Accel
    real(8), intent(out) :: KEnergy, PEnergy
    external :: CalcEnergyForces
    Pos = Pos + dt * Vel + 0.5 * dt*dt * Accel
    Vel = Vel + 0.5 * dt * Accel
    call CalcEnergyForces(Pos,ChargeList, M, L, rc, kappa, lambda, PEnergy, Accel, Dim, NAtom)
    Vel = Vel + 0.5 * dt * Accel
    KEnergy = 0.5 * sum(Vel*Vel)
end subroutine

subroutine CalcMIPD(Pos, L, Dim, NAtom, d)
    ! MIPD = Mean Interparticle Distance    
    implicit none
    integer, intent(in) :: Dim, NAtom
    real(8), intent(in), dimension(0:NAtom-1, 0:Dim-1) :: Pos
    real(8), intent(in) :: L
    real(8), dimension(Dim) :: rij, Posi
    real(8), intent(out) :: d
    integer :: i, j
    
    d = 0.
    
    do i = 0, NAtom - 1
        !store Pos(i,:) in a temporary array for faster access in j loop
        Posi = Pos(i,:)
        do j = i + 1, NAtom - 1          
            rij = Pos(j,:) - Posi
            rij = rij - L * dnint(rij / L)
            d = d + sqrt(sum(rij * rij))
        enddo
    enddo
    d = d / (NAtom*(NAtom-1)/2)
end subroutine


subroutine CenterPos(Pos, L,Dim, NAtom)
    implicit none
    integer, intent(in) :: Dim, NAtom
    real(8), intent(in) :: L
    real(8), intent(inout), dimension(0:NAtom-1, 0:Dim-1) :: Pos
!f2py intent(in,out,inplace) :: Pos
    real(8), dimension(Dim) ::  Recenter
    integer :: i, j

    Recenter = Pos((NAtom-1),:)- (L)
    do i = 0, NAtom -1
    Pos(i,:) = Pos(i,:) - Recenter
    do j = 0, 2
    if (Pos(i,j) < 0.0) then
    Pos(i,j) = Pos(i,j) + L
    else if (Pos(i,j) > L) then
    Pos(i,j) = Pos(i,j) - L
    endif
    enddo
    enddo

end subroutine